<?php
declare(strict_types=1);

namespace Inti\Provider\Query;

/**
 * Enables providers to filter provided items.
 *
 * @package Inti\Provider\Query
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
interface QueryInterface
{
    /**
     * Initializes the query.
     *
     * @return $this
     */
    public function query(): self;

    /**
     * Filter the items by an id.
     *
     * @param int $id
     * @return $this
     */
    public function id(int $id): self;

    /**
     * Filter the items by an array of ids.
     *
     * @param array $ids
     * @return $this
     */
    public function ids(array $ids): self;

    /**
     * Returns the number of items selected by the query.
     *
     * @return int
     */
    public function getCount(): int;

    /**
     * Returns one item selected by the query.
     *
     * @return array|null
     */
    public function getItem(): ?array;

    /**
     * Returns the items selected by the query.
     *
     * @return array
     */
    public function getItems(): array;
}
