<?php
declare(strict_types=1);

namespace Inti\Provider\Query;

use function ceil;
use function intval;

/**
 * Trait QueryPaginateTrait
 *
 * @package Inti\Provider\Query
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
trait QueryPaginateTrait
{
    use QueryTrait;

    public function paginate(int $page = 1, int $itemsPerPage = 20): QueryPaginateInterface
    {
        $this->assertQueryIsInitialized();
        $this->_query['paginate'] = [$page, $itemsPerPage];
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this;
    }

    public function getPageCount(): int
    {
        if (!isset($this->_query['paginate'])) {
            return 0;
        }

        [, $itemsPerPage] = $this->_query['paginate'];

        return intval(ceil($this->getCount() / $itemsPerPage));
    }
}
