<?php
declare(strict_types=1);

namespace Inti\Provider\Query;

use RuntimeException;

use function array_map;
use function get_class;

/**
 * Trait QueryTrait
 *
 * @package Inti\Provider\Query
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
trait QueryTrait
{
    protected array $_query;

    public function query(): QueryInterface
    {
        $this->_query = [];
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this;
    }

    public function id(int $id): QueryInterface
    {
        $this->assertQueryIsInitialized();
        $this->_query['ids'] = [$id];

        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this;
    }

    public function ids(array $ids): QueryInterface
    {
        if (!isset($this->_query)) {
            throw new RuntimeException(sprintf(
                "%s: Query is not initialized; You must call %s::query() before setting parameters.",
                __METHOD__, get_class($this)
            ));
        }

        $this->_query['ids'] = array_map('intval', $ids);

        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this;
    }

    /**
     * Extracts the id from the given item.
     *
     * @param array $item
     * @return int
     */
    protected function getId(array $item): int
    {
        if (!isset($this->field_map['id'])) {
            throw new RuntimeException(sprintf(
                "The provider '%s' must define an id field.",
                get_class($this)
            ));
        }

        if (!array_key_exists($this->field_map['id'], $item)) {
            throw new RuntimeException(sprintf(
                "%s: Item array does not have a key '%s'.",
                __METHOD__, $this->field_map['id']
            ));
        }

        return intval($item[$this->field_map['id']]);
    }

    protected function assertQueryIsInitialized(): void
    {
        if (!isset($this->_query)) {
            throw new RuntimeException(sprintf(
                "Query is not initialized; You must call %s::query() before setting parameters.",
                get_class($this)
            ));
        }
    }
}
