<?php
declare(strict_types=1);

namespace IntiTest\Provider;

use Inti\Provider\ProviderInterface;
use IntiTest\Provider\Mock;
use PHPUnit\Framework\TestCase;
use RuntimeException;

/**
 * Class AbstractProviderTest
 *
 * @package IntiTest\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 */
final class AbstractProviderTest extends TestCase
{
    /** @var ProviderInterface|Mock\MockProvider */
    private ProviderInterface $provider;

    protected function setUp(): void
    {
        $this->provider = new Mock\MockProvider();
    }

    public function testWillThrowExceptionIfQueryNotInitialized()
    {
        $this->expectException(RuntimeException::class);
        $this->provider->getItems();
    }

    public function testCanGetItems()
    {
        $items = $this->provider
            ->query()
            ->getItems();

        $this->assertCount(8, $items);
    }

    public function testCanGetItemsByIds()
    {
        $items = $this->provider
            ->query()
            ->ids([1, 3, 5, 6])
            ->getItems();

        $this->assertCount(4, $items);

        $item = $this->provider
            ->query()
            ->id(3)
            ->getItem();

        $this->assertIsArray($item);
        $this->assertEquals(3, $item['id']);
    }

    public function testCanCountItems()
    {
        $this->assertEquals(8, $this->provider->query()->getCount());
    }

    public function testCanPaginateResults()
    {
        $this->provider->query();
        $this->provider->paginate(3, 2);
        $items = $this->provider->getItems();

        $this->assertCount(2, $items);
        $this->assertEquals(5, $items[0]['id']);
        $this->assertEquals(6, $items[1]['id']);

        $this->assertEquals(8, $this->provider->getCount());
        $this->assertEquals(4, $this->provider->getPageCount());
    }
}
