<?php
declare(strict_types=1);

namespace IntiTest\Provider\Mock;

use Inti\Provider\AbstractProvider;
use Inti\Provider\Query\QueryPaginateInterface;
use Inti\Provider\Query\QueryPaginateTrait;
use Inti\Provider\Type\TypeInterface;

use function array_chunk;
use function array_filter;
use function count;
use function in_array;

/**
 * Class MockProvider
 *
 * @package IntiTest\Provider\Mock
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
class MockProvider extends AbstractProvider implements
    QueryPaginateInterface
{
    use QueryPaginateTrait;

    private array $items = [
        ['id' => 1, 'title' => 'First item', 'slug' => 'first-item'],
        ['id' => 2, 'title' => 'Second item', 'slug' => 'second-item'],
        ['id' => 3, 'title' => 'Third item', 'slug' => 'third-item'],
        ['id' => 4, 'title' => 'Fourth item', 'slug' => 'fourth-item'],
        ['id' => 5, 'title' => 'Fifth item', 'slug' => 'fifth-item'],
        ['id' => 6, 'title' => 'Sixth item', 'slug' => 'sixth-item'],
        ['id' => 7, 'title' => 'Seventh item', 'slug' => 'seventh-item'],
        ['id' => 8, 'title' => 'Eight item', 'slug' => 'eight-item'],
    ];

    public function getProvidedType(): TypeInterface
    {
        return MockType::instance();
    }

    protected function fetchResults(): array
    {
        $results = array_filter($this->items, function (array $item) {
            if (isset($this->_query['ids']) && !empty($this->_query['ids'])) {
                $itemId = $this->getId($item);
                return in_array($itemId, $this->_query['ids']);
            }

            return true;
        });

        if (isset($this->_query['paginate'])) {
            [$page, $itemsPerPage] = $this->_query['paginate'];
            $results = array_chunk($results, $itemsPerPage);
            $results = $results[$page - 1];
        }

        return $results;
    }

    public function getCount(): int
    {
        return count($this->items);
    }
}
