<?php
declare(strict_types=1);

namespace IntiTest\Provider\Service;

use Inti\Provider\Service\ProviderManager;
use IntiTest\Provider\Mock;
use Laminas\ServiceManager\ServiceManager;
use PHPUnit\Framework\TestCase;

/**
 * Class ProviderManagerTest
 *
 * @package IntiTest\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
final class ProviderManagerTest extends TestCase
{
    protected function createManager(bool $autoregister_providers = false, array $registered = []): ProviderManager
    {
        return new ProviderManager(new ServiceManager(), [
            'autoregister_providers' => $autoregister_providers,
            'registered' => $registered,

            'factories' => [
                'custom_type' => Mock\CustomProviderFactory::class,
                'other_type'  => Mock\CustomProviderFactory::class,
            ],
            'invokables' => [
                Mock\MockProvider::class,
            ],
        ]);
    }

    public function testCanGetAllProviders()
    {
        $manager = $this->createManager(true);
        $providers = $manager->getProviders();

        $this->assertCount(3, $providers);
    }

    public function testCanGetProvidersForType()
    {
        $manager = $this->createManager(true);

        $this->assertCount(1, $manager->getProviders(Mock\CustomType::instance()));
    }

    public function testCanGetProviderForType()
    {
        $manager = $this->createManager(true);

        $this->assertTrue($manager->canProvide(Mock\CustomType::instance()));
        $this->assertTrue($manager->canProvide(Mock\CustomType::instance('other_type')));
        $this->assertNull($manager->getProvider(Mock\CustomType::instance('not_provided')));
    }

    public function testCanRegisterProviders()
    {
        $custom_type = Mock\CustomType::instance();
        $other_type = Mock\CustomType::instance('other_type');

        $manager = $this->createManager();
        $this->assertFalse($manager->canProvide($custom_type));
        $this->assertFalse($manager->canProvide($other_type));

        $manager->registerProviders(['custom_type', 'other_type']);

        $this->assertTrue($manager->canProvide($custom_type));
        $this->assertTrue($manager->canProvide($other_type));
    }

    public function testCanRegisterInstances()
    {
        $manager = $this->createManager();
        $this->assertFalse($manager->canProvide(Mock\MockType::instance()));

        $manager->registerProvider(new Mock\MockProvider());
        $this->assertTrue($manager->canProvide(Mock\MockType::instance()));
    }
}
